package com.example.flutter_bl_umpush;

import android.app.Notification;
import android.content.Context;
import android.content.Intent;
import android.os.Handler;
import android.util.Log;
import android.widget.RemoteViews;
import android.widget.Toast;

import com.umeng.analytics.MobclickAgent;
import com.umeng.commonsdk.UMConfigure;
import com.umeng.message.IUmengRegisterCallback;
import com.umeng.message.MsgConstant;
import com.umeng.message.PushAgent;
import com.umeng.message.UTrack;
import com.umeng.message.UmengMessageHandler;
import com.umeng.message.UmengNotificationClickHandler;
import com.umeng.message.entity.UMessage;

import io.flutter.app.FlutterApplication;

import static android.content.ContentValues.TAG;

public class UmApplication extends FlutterApplication {

    public static final String UPDATE_STATUS_ACTION = "com.example.bl.umpush.action.UPDATE_STATUS";
    private Handler handler;

    @Override
    public void onCreate() {
        super.onCreate();

        UMConfigure.setLogEnabled(true);
        UMConfigure.init(getApplicationContext(), "5de487310cafb221820010c2",
                "Umeng", UMConfigure.DEVICE_TYPE_PHONE, "0a4c23cee3d2d6f692341b14429044ed");

        //统计SDK是否支持采集在子进程中打点的自定义事件，默认不支持
        UMConfigure.setProcessEvent(true);//支持多进程打点
        //sdk开启通知声音
        PushAgent mPushAgent = PushAgent.getInstance(getApplicationContext());
        mPushAgent.setNotificationPlaySound(MsgConstant.NOTIFICATION_PLAY_SDK_ENABLE);

        //注册推送服务 每次调用register都会回调该接口
        mPushAgent.register(new IUmengRegisterCallback() {
            @Override
            public void onSuccess(String deviceToken) {
                Util.token = deviceToken;
                Logger.i(TAG, "注册推送服务 device token: " + deviceToken);
                FlutterBlUmpushPlugin.channel.invokeMethod("connectionChange", Util.token != null && Util.token.length() > 0);

                FlutterBlUmpushPlugin.channel.invokeMethod("networkDidLogin", Util.token);
            }

            @Override
            public void onFailure(String s, String s1) {
                Util.token = null;
                FlutterBlUmpushPlugin.channel.invokeMethod("connectionChange", Util.token != null && Util.token.length() > 0);
                Logger.i(TAG, "register failed: " + s + " " + s1);
            }
        });

        // 通知的回调方法（通知送达时会回调）
        UmengMessageHandler messageHandler = new UmengMessageHandler() {
            @Override
            public void dealWithNotificationMessage(Context context, UMessage msg) {
                //调用super，会展示通知，不调用super，则不展示通知。
                try {
                    if (!Logger.SHUTDOWNLOG) {
                        Logger.i(TAG, "收到推送下来的通知: " + msg.custom);
                        Logger.i(TAG, "extras: " + msg.extra.toString());
                    }
                    FlutterBlUmpushPlugin.channel.invokeMethod("receivePushMsg", msg);
                } catch (Throwable e) {
                    e.printStackTrace();
                }
                super.dealWithNotificationMessage(context, msg);
            }

            /** 自定义消息的回调方法  */
            @Override
            public void dealWithCustomMessage(final Context context, final UMessage msg) {
                handler.post(new Runnable() {

                    @Override
                    public void run() {
                        // 对自定义消息的处理方式，点击或者忽略
                        try {
                            if (!Logger.SHUTDOWNLOG) {
                                Logger.i(TAG, "收到自定义消息:" + msg.custom);
                            }
                        } catch (Throwable e) {
                            e.printStackTrace();
                        }
                    }
                });
            }

            /** 自定义通知栏样式的回调方法 */
            @Override
            public Notification getNotification(Context context, UMessage msg) {
                return super.getNotification(context, msg);
            }
        };
        mPushAgent.setMessageHandler(messageHandler);

        /**
         * 自定义行为的回调处理，参考文档：高级功能-通知的展示及提醒-自定义通知打开动作
         * UmengNotificationClickHandler是在BroadcastReceiver中被调用，故
         * 如果需启动Activity，需添加Intent.FLAG_ACTIVITY_NEW_TASK
         * */
        UmengNotificationClickHandler notificationClickHandler = new UmengNotificationClickHandler() {
            @Override
            public void launchApp(Context context, UMessage msg) {
                super.launchApp(context, msg);
            }

            @Override
            public void openUrl(Context context, UMessage msg) {
                super.openUrl(context, msg);
            }

            @Override
            public void openActivity(Context context, UMessage msg) {
                super.openActivity(context, msg);
            }

            @Override
            public void dealWithCustomAction(Context context, UMessage msg) {
                Toast.makeText(context, msg.custom, Toast.LENGTH_SHORT).show();
            }
        };
        mPushAgent.setNotificationClickHandler(notificationClickHandler);

        //使用完全自定义处理
        //mPushAgent.setPushIntentServiceClass(UmengNotificationService.class);
        //小米通道
        //MiPushRegistar.register(this, XIAOMI_ID, XIAOMI_KEY);
        //华为通道
        //HuaWeiRegister.register(this);

        // true表示打开错误统计功能，false表示关闭 默认为打开
        MobclickAgent.setCatchUncaughtExceptions(true);

        // 页面采集的两种模式：AUTO和MANUAL，Android 4.0及以上版本使用AUTO,4.0以下使用MANUAL
        MobclickAgent.setPageCollectionMode(MobclickAgent.PageMode.AUTO);
//        int results = call.argument("mode");
//        if (android.os.Build.VERSION.SDK_INT >= Build.VERSION_CODES.KITKAT) {
//            if (results == 0) {
//                // 大于等于4.4选用AUTO页面采集模式
//                MobclickAgent.setPageCollectionMode(MobclickAgent.PageMode.AUTO);
//            }
//        } else if (results == 1) {
//            MobclickAgent.setPageCollectionMode(MobclickAgent.PageMode.MANUAL);
//        }
    }
}