package com.honghu.topsports;

import android.app.Application;
import android.app.Notification;
import android.content.Context;
import android.content.Intent;
import android.net.Uri;
import android.os.Bundle;
import android.widget.Toast;

import com.google.gson.Gson;
import com.honghu.topsports.Event.ReceiveUmPushEvent;
import com.honghu.topsports.Event.RegistUmPushEvent;
import com.honghu.topsports.model.FreeMessageModel;
import com.honghu.topsports.umeng.FlutterBlUmpushPlugin;
import com.umeng.analytics.MobclickAgent;
import com.umeng.commonsdk.UMConfigure;
import com.umeng.message.IUmengRegisterCallback;
import com.umeng.message.MsgConstant;
import com.umeng.message.PushAgent;
import com.umeng.message.UmengMessageHandler;
import com.umeng.message.UmengNotificationClickHandler;
import com.umeng.message.entity.UMessage;

import org.android.agoo.huawei.HuaWeiRegister;
import org.android.agoo.oppo.OppoRegister;
import org.android.agoo.xiaomi.MiPushRegistar;
import org.greenrobot.eventbus.EventBus;

import java.util.HashMap;
import java.util.Map;

import io.flutter.Log;
import io.flutter.app.FlutterApplication;
import io.flutter.plugin.common.PluginRegistry;

public class MyApplication extends FlutterApplication implements PluginRegistry.PluginRegistrantCallback {

    public static final String TAG = "MyApplication";
    public static String myDeviceToken;

    @Override
    public void onCreate() {
        super.onCreate();

        UMConfigure.setLogEnabled(false);
        UMConfigure.init(getApplicationContext(), "",
                "Umeng", UMConfigure.DEVICE_TYPE_PHONE, "");

        //统计SDK是否支持采集在子进程中打点的自定义事件，默认不支持
        UMConfigure.setProcessEvent(true);//支持多进程打点
        //sdk开启通知声音
        PushAgent mPushAgent = PushAgent.getInstance(getApplicationContext());
        mPushAgent.setNotificationPlaySound(MsgConstant.NOTIFICATION_PLAY_SDK_ENABLE);

        //注册推送服务 每次调用register都会回调该接口
        mPushAgent.register(new IUmengRegisterCallback() {
            @Override
            public void onSuccess(String deviceToken) {
                myDeviceToken = deviceToken;
                Log.i(TAG, "注册推送服务 onSuccess:  device token " + deviceToken);
//                FlutterBlUmpushPlugin.channel.invokeMethod("callbackDeviceToken", MyApplication.myDeviceToken);
                RegistUmPushEvent registUmPushEvent = new RegistUmPushEvent(0, "deviceToken");
                EventBus.getDefault().post(registUmPushEvent);
                Log.i(TAG, "注册推送时间: " + System.currentTimeMillis());
            }

            @Override
            public void onFailure(String s, String s1) {
                Log.i(TAG, "注册推送服务 Fail !");
            }
        });

        // 通知的回调方法（通知送达时会回调）
        UmengMessageHandler messageHandler = new UmengMessageHandler() {
            @Override
            public void dealWithNotificationMessage(Context context, UMessage msg) {
                //调用super，会展示通知，不调用super，则不展示通知。
                super.dealWithNotificationMessage(context, msg);
                Log.i(TAG, "UmengMessageHandler -> dealWithNotificationMessage: 收到消息：" + msg.text);
                // 设置角标
                setBadgeNum(1);
                setBadgeOfSumsung(1);
            }

            /** 自定义消息的回调方法  */
            @Override
            public void dealWithCustomMessage(final Context context, final UMessage msg) {
                Log.i(TAG, "UmengMessageHandler -> dealWithCustomMessage: 收到自定义消息：" + msg.custom);
                // 设置角标
                setBadgeNum(1);
                setBadgeOfSumsung(1);
            }

            /** 自定义通知栏样式的回调方法 */
            @Override
            public Notification getNotification(Context context, UMessage msg) {
                return super.getNotification(context, msg);
            }
        };
        mPushAgent.setMessageHandler(messageHandler);

        /**
         * 自定义行为的回调处理，参考文档：高级功能-通知的展示及提醒-自定义通知打开动作
         * UmengNotificationClickHandler是在BroadcastReceiver中被调用，故
         * 如果需启动Activity，需添加Intent.FLAG_ACTIVITY_NEW_TASK
         * */
        UmengNotificationClickHandler notificationClickHandler = new UmengNotificationClickHandler() {
            @Override
            public void launchApp(Context context, UMessage msg) {
                super.launchApp(context, msg);
            }

            @Override
            public void openUrl(Context context, UMessage msg) {
                super.openUrl(context, msg);
            }

            @Override
            public void openActivity(Context context, UMessage msg) {
                super.openActivity(context, msg);
            }

            @Override
            public void dealWithCustomAction(Context context, UMessage msg) {

                // 解析自定义消息
                String msgJson = msg.custom;
                if (msgJson.isEmpty()) {
                    Log.i(TAG, "dealWithCustomAction: 自定义消息为空！custom:" + msg.custom);
                    return;
                }
                // 解析Json
                Gson gson = new Gson();
                FreeMessageModel freeMessageModel = gson.fromJson(msgJson, FreeMessageModel.class);
                if (null == freeMessageModel) {
                    Log.i(TAG, "dealWithCustomAction: 解析自定义消息失败！" + msg.custom);
                    return;
                }

                // 启动App
                Intent newIntent = new Intent(context, MainActivity.class);
                // 在Service中启动Activity，必须设置如下标志
                newIntent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
                context.startActivity(newIntent);

                // 取值组装
                Map map = new HashMap();
                map.put("type", freeMessageModel.getType());
                map.put("redirectValue", freeMessageModel.getRedirectValue());
                map.put("msgId", freeMessageModel.getMsgId());
                map.put("paraOne", freeMessageModel.getParaOne());
                map.put("paraTwo", freeMessageModel.getParaTwo());
                map.put("paraThree", freeMessageModel.getParaThree());
                // 发送Event
                ReceiveUmPushEvent receiveUmPushEvent = new ReceiveUmPushEvent(0, map);
                EventBus.getDefault().post(receiveUmPushEvent);

                Log.i(TAG, "收到推送->发送消息: " + msg.custom);
            }
        };
        mPushAgent.setNotificationClickHandler(notificationClickHandler);

        //小米通道
        MiPushRegistar.register(getApplicationContext(), "", "");
        //华为通道
        HuaWeiRegister.register(this);
//        //vivo 通道
//        VivoRegister.register(this);
        //OPPO通道，参数1为app key，参数2为app secret
        OppoRegister.register(this, "", "");

        // true表示打开错误统计功能，false表示关闭 默认为打开
        MobclickAgent.setCatchUncaughtExceptions(false);

        // 页面采集的两种模式：AUTO和MANUAL，Android 4.0及以上版本使用AUTO,4.0以下使用MANUAL
        MobclickAgent.setPageCollectionMode(MobclickAgent.PageMode.AUTO);
//        int results = call.argument("mode");
//        if (android.os.Build.VERSION.SDK_INT >= Build.VERSION_CODES.KITKAT) {
//            if (results == 0) {
//                // 大于等于4.4选用AUTO页面采集模式
//                MobclickAgent.setPageCollectionMode(MobclickAgent.PageMode.AUTO);
//            }
//        } else if (results == 1) {
//            MobclickAgent.setPageCollectionMode(MobclickAgent.PageMode.MANUAL);
//        }
    }

    /**
     * 华为角标设置
     * @param count
     */
    private void setBadgeNum(int count) {
        try {
            Bundle bundle = new Bundle();
            bundle.putString("package", getPackageName()); // com.test.badge is your package name
            bundle.putString("class",  getPackageName() + ".MainActivity"); // com.test. badge.MainActivity is your apk main activity
            bundle.putInt("badgenumber", count);
            this.getContentResolver().call(Uri.parse("content://com.huawei.android.launcher.settings/badge/"),
                    "change_badge", null, bundle);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * 三星角标设置
     * @param count   count
     */
    private void setBadgeOfSumsung(int count) {
        Intent intent = new Intent("android.intent.action.BADGE_COUNT_UPDATE");
        intent.putExtra("badge_count", count);
        intent.putExtra("badge_count_package_name", getPackageName());
        intent.putExtra("badge_count_class_name", getPackageName() + ".MainActivity");
        sendBroadcast(intent);
    }

    @Override
    public void registerWith(PluginRegistry registry) {

    }

}
